from datetime import datetime


class ConsoleLogger:
    def __init__(self):
        self.start_dt = datetime.utcnow()

    def gray_text(self, message: str):
        return f'\033[02m{message}\033[0m'

    def green_text(self, message: str):
        return f'\033[32m{message}\033[0m'

    def yellow_text(self, message: str):
        return f'\033[93m{message}\033[0m'

    def red_text(self, message: str):
        return f'\033[31m{message}\033[0m'

    def log(self, message):
        delta = datetime.utcnow() - self.start_dt

        print(f'{self.gray_text(f"[{delta}]")} {message}')

    def info(self, message):
        self.log(self.gray_text(message))

    def error(self, message):
        self.log(self.red_text(message))

    def warning(self, message):
        self.log(self.yellow_text(message))

    def success(self, message):
        self.log(self.green_text(message))



import json

from datetime import date, datetime


class EyjaJSONEncoder(json.JSONEncoder):
  def default(self, obj):
    if isinstance(obj, (date, datetime)):
      return obj.strftime('%Y-%m-%d %H:%M:%S.%f')



import aiofiles
import yaml

from typing import Union

from eyja.hubs.base_hub import BaseHub


class ConfigHub(BaseHub):
    _config: dict = {}

    @classmethod
    async def init(cls, config_file: str = None, config: Union[str, dict] = None) -> bool:
        init_result = False
        if config_file:
            init_result = await cls.load_from_file(config_file)
        elif config:
            if isinstance(config, str):
                init_result = cls.load_from_str(config)
            elif isinstance(config, dict):
                init_result = cls.load_from_dict(config)

        if init_result:
            await super().init()

        return init_result

    @classmethod
    async def reset(cls):
        cls._config = {}

        await super().reset()

    @classmethod
    async def load_from_file(cls, config_file: str = None) -> bool:
        async with aiofiles.open(config_file, 'r') as fp:
            config_data = await fp.read()

        return cls.load_from_str(config_data)

    @classmethod
    def load_from_str(cls, data: str) -> bool:
        config_data = yaml.load(data, Loader=yaml.FullLoader)
        return cls.load_from_dict(config_data)        

    @classmethod
    def load_from_dict(cls, data: dict) -> bool:
        cls._config = data

        return True

    @classmethod
    def get(cls, path: str) -> Union[dict, None]:
        path_parts = path.split('.')
        result = None

        for part in path_parts:
            if not result:
                result = cls._config
            
            if not part in result:
                return None

            result = result.get(part)
        
        return result



import random
import string


def random_string(prefix = None, length=0):
    result = prefix if prefix else ''

    letters = string.ascii_lowercase + string.digits
    return f'{result}{"".join(random.choice(letters) for i in range(length-len(result)))}'



from pydantic import BaseModel


class BaseDataModel(BaseModel):
    def dict(self, **kwargs):
        hidden_fields = [field for field in list(self.__fields__.keys()) if field[0] == '_']
        kwargs.setdefault("exclude", hidden_fields)
        return super().dict(**kwargs)



class BaseStorageClient:
    _config_cls = dict
    _config = None
    _buckets = []

    def __init__(self, config: dict):
        self._config = self._config_cls(config)

    async def init(self):
        pass

    async def save(self, obj, object_space, object_type):
        pass

    async def delete(self, obj, object_space, object_type):
        pass

    async def delete_all(self, obj, object_space, object_type, filter):
        pass

    async def get(self, obj_cls, object_space, object_type, object_id):
        pass

    async def find(self, obj_cls, object_space, object_type, filter):
        pass



import os

from unittest import IsolatedAsyncioTestCase

from eyja.main import Eyja
from eyja.hubs.config_hub import ConfigHub
from eyja.hubs.plugin_hub import PluginHub
from eyja.hubs.data_hub import DataHub


class EyjaTest(IsolatedAsyncioTestCase):
    str_config = '''
        storages:
    '''

    dict_config = {
        'storages': {}
    }

    file_config = 'tests/config/simple_test.yml'

    async def test_eyja_init_without_config(self):
        await Eyja.reset()
        await Eyja.init()

        self.assertFalse(Eyja.is_initialized())

    async def test_eyja_init_with_str_config(self):
        await Eyja.reset()
        await Eyja.init(config=self.str_config)

        self.assertTrue(Eyja.is_initialized())

    async def test_eyja_init_with_dict_config(self):
        await Eyja.reset()
        await Eyja.init(config=self.dict_config)

        self.assertTrue(Eyja.is_initialized())

    async def test_eyja_init_with_file_config(self):
        await Eyja.reset()
        await Eyja.init(config_file=os.path.join(os.getcwd(), self.file_config))

        self.assertTrue(Eyja.is_initialized())



from .hub_errors import (
    LoadConfigError,
    WrongConnectionError,
)
from .model_errors import (
    ParseModelNamespaceError,
    MissedRepresentationError
)


__all__ = [
    'LoadConfigError',
    'ParseModelNamespaceError',
    'WrongConnectionError',
    'MissedRepresentationError',
]



class BaseHub:
    _initialized = False

    @classmethod
    async def init(cls):
        cls._initialized = True

    @classmethod
    async def reset(cls):
        cls._initialized = False

    @classmethod
    def is_initialized(cls):
        return cls._initialized
